<?php

    // Считать 1 байт с пакета
    function get_byte ($data)
    {
        return ord($data[0]);
    }

    // Считать 8 байт с пакета и вернуть как число
    function get_double ($data)
    {
        // Распаковываем
        $arr = unpack('d1in', substr($data, 0, 8));

        return (int) $arr['in'];
    }

    // Считать 4 байта с пакета и вернуть как число
    function get_int ($data)
    {
        // Распаковываем
        $arr = unpack('l1in', substr($data, 0, 4));

        return (int) $arr['in'];
    }

    // Считать 2 байта с пакета и вернуть как число
    function get_short ($data)
    {
        // Распаковываем
        $arr = unpack('S1in', substr($data, 0, 2));

        return (int) $arr['in'];
    }

    // Считать текст с пакета (текст - '4 байта длина текста + N байт сам текст')
    function get_string ($data)
    {
        // Распаковываю длину текста
        $arr = unpack('l1len', substr($data, 0, 4));

        // Не хватает для считывания текста
        if (strlen($data) <  4 + $arr['len'] * 2)
        {
            outtext('Функция `get_string` (' . $arr['len'] . '). Не хватает байтов для считывания текста: ' . strlen($data) . '/' . (4 + $arr['len'] * 2));

            return '';
        }

        return iconv('UTF-16LE', 'UTF-8', mb_substr($data, 4, $arr['len'] * 2));
    }


    // Считать 1 байт с пакета
    function get_byte_c (&$data)
    {
        $return = ord($data[0]);

        $data = substr($data, 1);

        return $return;
    }

    // Считать 8 байт с пакета и вернуть как число
    function get_double_c (&$data)
    {
        // Распаковываем
        $arr = unpack('d1in', substr($data, 0, 8));

        // Обрезаем пакет
        $data = substr($data, 8);

        return (int) $arr['in'];
    }

    // Считать 8 байт с пакета и вернуть число с плавающей запятой
    function get_time(&$data)
    {
        // Распаковываем
        $arr = unpack('d1in', substr($data, 0, 8));

        // Обрезаем пакет
        $data = substr($data, 8);

        return $arr['in'];
    }

    // Считать 4 байта с пакета и вернуть как число
    function get_int_c (&$data)
    {
        // Распаковываем
        $arr = unpack('l1in', substr($data, 0, 4));

        // Обрезаем пакет
        $data = substr($data, 4);

        return (int) $arr['in'];
    }

    // Считать 2 байта с пакета и вернуть как число
    function get_short_c (&$data)
    {
        // Распаковываем
        $arr = unpack('S1in', substr($data, 0, 2));

        // Обрезаем пакет
        $data = substr($data, 2);

        return (int) $arr['in'];
    }

    // Считать текст с пакета (текст - '4 байта длина текста + N байт сам текст')
    function get_string_c (&$data)
    {
        // Распаковываю
        $arr = unpack('l1len', substr($data, 0, 4));

        // Не хватает для считывания текста
        if (strlen($data) <  4 + $arr['len'] * 2)
        {
            outtext('Функция `get_string_c` (' . $arr['len'] . '). Не хватает байтов для считывания текста: ' . strlen($data) . '/' . (4 + $arr['len'] * 2));

            return '';
        }

        $return = substr($data, 4, $arr['len'] * 2);

        $data = substr($data, 4 + $arr['len'] * 2);

        return iconv('UTF-16LE', 'UTF-8', $return);
    }

    // Считать текст с пакета (текст - '8 байта длина текста + N байт сам текст')
    function get_string_c1 (&$data)
    {
        // Распаковываем
        $arr = unpack('d1in', substr($data, 0, 8));

        // Обрезаем пакет
        $data = substr($data, 8);

        return $arr['in'];

    }

    function get_data_c (&$data)
    {
        // Распаковываю
        $arr = unpack('l1len', substr($data, 0, 4));

        // Не хватает для считывания текста
        if (strlen($data) <  4 + $arr['len'] * 2)
        {
            outtext('Функция `get_string_c` (' . $arr['len'] . '). Не хватает байтов для считывания текста: ' . strlen($data) . '/' . (4 + $arr['len'] * 2));

            return '';
        }

        $return = substr($data, 4, $arr['len'] * 2);

        $data = substr($data, 4 + $arr['len'] * 2);

        return $return;
    }



    // Упаковать число в 1 байт
    function pack_byte ($num)
    {
        return pack('C', $num);
    }

    // Упаковать число в 8 байтовое double
    function pack_double ($num)
    {
        return pack('d', $num);
    }

    // Упаковать число в 4х байтовое беззнаковое
    function pack_int ($num)
    {
        return pack('l', $num);
    }

    // Упаковать число в 2х байтовое беззнаковое
    function pack_short ($num)
    {
        return pack('S', $num);
    }

    // Упаковать текст (длина + UTF текст)
    function pack_text ($text)
    {
        return pack('l', mb_strlen($text)) . iconv('UTF-8', 'UTF-16LE', $text);
    }

    // Упаковать картинку (длина + дата картинки)
    function pack_picture ($data)
    {
        return pack('l', strlen($data)) . $data;
    }

    // Упаковать хэш авторизации (текст)
    function pack_hash ($data)
    {
        return iconv('UTF-8', 'UTF-16LE//IGNORE', $data);
    }


    // Подключение библиотек
    function load_ext ($name)
    {
        $name = 'php_' . $name;

        // Уже загружена
        if (extension_loaded($name)) return true;

        if (!dl($name . '.' . PHP_SHLIB_SUFFIX))
        {
            outtext('Не удалось загрузить расширение - <' . $name . '.' . PHP_SHLIB_SUFFIX . '>, проверьте наличие в папке php/ext');

            return false;
        }

        outtext('<' . $name . '.' . PHP_SHLIB_SUFFIX . '> успешно загружен');

        return true;
    }

    function get_socket_error ()
    {
        return '(№' . socket_last_error() . ': ' . iconv('cp1251', 'utf-8', rtrim(socket_strerror(socket_last_error()))) . ')';
    }

    // Обработка строки "!$cmd $data"
    function parse_cmd ($text, &$cmd, &$data, $multiline = false)
    {
        $text = trim($text);

        if (mb_substr($text, 0, 1) != '!') return false;

        // Берем в расчет только первую строчку
        if (!$multiline)
        {
            $one_string = mb_split("/\r\n/", $text);
            if (isset($one_string[0])) $text = $one_string[0];
        }

        // Ищем в сообщении шаблон команды (!команда аргументы)
        preg_match('/^!([_\pL\pN]+)\s?(.+)?/u', $text, $match);
        $cmd = '';
        $data = '';

        if (!isset($match[1])) return false;

        $cmd = mb_strtolower($match[1]);

        if (isset($match[2])) $data = $match[2];

        return true;
    }

    // Вывод текста в CMD
    function outtext ($text, $time = true, $enc = true, $nl = true)
    {
        /*
        $fp = fopen('2.txt','a');
        $x = ($time ? date('d/m H:i:s | ') : '') . ($enc ? iconv('utf-8', 'cp866//IGNORE', $text) : $text) . ($nl ? PHP_EOL : '');
        fwrite($fp,$x);
        fclose($fp);
        */
    }

    // Ввод с консоли
    function input($len = 255)
    {
        $f = fopen('php://stdin', 'r');
        $input = trim(fgets(STDIN));
        fclose($f);

        return $input;
    }

    // Перевод секунд в читабельный вид (Xd Xh Xm Xs)
    function str_time ($sec)
    {
        $sec = (int) $sec;

        if ($sec >= 60 && $sec < 3600)
        {
            $min = floor($sec / 60);
            $sec -= $min * 60;
        }
        elseif ($sec >= 3600 && $sec < 86400)
        {
            $hour = floor($sec / 3600);
            $sec -= $hour * 3600;

            $min = floor($sec / 60);
            $sec -= $min * 60;
        }
        elseif ($sec >= 86400)
        {
            $day = floor($sec / 86400);
            $sec -= $day * 86400;

            $hour = floor($sec / 3600);
            $sec -= $hour * 3600;

            $min = floor($sec / 60);
            $sec -= $min * 60;
        }

        return (isset($day) ? $day . 'd ' : '') . (isset($hour) ? $hour . 'h ' : '') . (isset($min) ? $min . 'm ' : '') . $sec . 's';
    }

    // Перевод байтов в читабельный вид
    function str_size ($bait)
    {
        $bait = (int) $bait;

        return ($bait >= 1048576 ? floor(1000*$bait/1048576)/1000 . ' MB' : ($bait >= 1024 ? floor(1000*$bait/1024)/1000 . ' KB' : $bait . ' B'));
    }

    function packet_dump ($packet, $send = false, $col = 7)
    {
        echo '[ ' . str_repeat('-', 40) . PHP_EOL . '   ' . ($send ? 'Send to' : 'Get from') . ' server ' . strlen($packet) . 'b' . PHP_EOL . '   ';

        for ($i = 1; $i <= strlen($packet); $i++)
        {
            $_d = ord($packet[$i-1]);
            $_n = dechex($_d);

            if (strlen($_n) == 1) $_n = '0' . $_n;

            if (strlen($_d) == 1) $_d = '  ' . $_d;
            elseif (strlen($_d) == 2) $_d = ' ' . $_d;

            echo  "$_n($_d)  ";

            if ($i % $col == 0) echo "\n   ";
        }
        echo PHP_EOL . PHP_EOL . '> ' . $packet . PHP_EOL . str_repeat('-', 50) . ']' . PHP_EOL;
    }

    function Resize($w,$h,$path,$name,$types,$parametr)
    {
        $types = strtolower($types);
        $file_name = $name;
        $smallimage = $path;
        $ratio = $w/$h;
        $size_img = getimagesize($file_name);
        $src_ratio = $size_img[0]/$size_img[1];
        if($parametr == 1) 
        {
            if($size_img[0] > $w || $size_img[1] > $h)
            {
                if($ratio<$src_ratio) $h = $w/$src_ratio; else $w = $h*$src_ratio;
            }
            else
            {
                $w = $size_img[0];
                $h = $size_img[1];
            }
        }
        $dest_img = imagecreatetruecolor($w,$h);
        if($types == "png")
        {
            $src_img = imagecreatefrompng($file_name);
            imagesavealpha($dest_img, true);
            $png=imagecolorallocatealpha($dest_img,0,0,0,127);
            imagefill($dest_img, 0, 0, $png);
        }
        if($types == "jpg" || $types == "jpeg") $src_img = imagecreatefromjpeg($file_name);
        if($types == "gif")
        {
            $src_img = imagecreatefromgif($file_name);
            $gif = imagecolorat($dest_img,0,0);
            imagecolortransparent($dest_img,$gif);
        }
        if(!imagecopyresampled($dest_img,$src_img,0,0,0,0,$w,$h,$size_img[0],$size_img[1])) return false;
        if($types == "png") imagepng($dest_img,$smallimage);
        if($types == "jpg" || $types == "jpeg") imagejpeg($dest_img,$smallimage,50);
        if($types == "gif") imagegif($dest_img,$smallimage);
        imagedestroy($dest_img);
        imagedestroy($src_img);
    }

    function JsonDecodeUser($json)
    {
        $json = json_encode($json);
        settype($json, "string");
        $cyr_chars = array (
            '\u0430' => 'а', '\u0410' => 'А',
            '\u0431' => 'б', '\u0411' => 'Б',
            '\u0432' => 'в', '\u0412' => 'В',
            '\u0433' => 'г', '\u0413' => 'Г',
            '\u0434' => 'д', '\u0414' => 'Д',
            '\u0435' => 'е', '\u0415' => 'Е',
            '\u0451' => 'ё', '\u0401' => 'Ё',
            '\u0436' => 'ж', '\u0416' => 'Ж',
            '\u0437' => 'з', '\u0417' => 'З',
            '\u0438' => 'и', '\u0418' => 'И',
            '\u0439' => 'й', '\u0419' => 'Й',
            '\u043a' => 'к', '\u041a' => 'К',
            '\u043b' => 'л', '\u041b' => 'Л',
            '\u043c' => 'м', '\u041c' => 'М',
            '\u043d' => 'н', '\u041d' => 'Н',
            '\u043e' => 'о', '\u041e' => 'О',
            '\u043f' => 'п', '\u041f' => 'П',
            '\u0440' => 'р', '\u0420' => 'Р',
            '\u0441' => 'с', '\u0421' => 'С',
            '\u0442' => 'т', '\u0422' => 'Т',
            '\u0443' => 'у', '\u0423' => 'У',
            '\u0444' => 'ф', '\u0424' => 'Ф',
            '\u0445' => 'х', '\u0425' => 'Х',
            '\u0446' => 'ц', '\u0426' => 'Ц',
            '\u0447' => 'ч', '\u0427' => 'Ч',
            '\u0448' => 'ш', '\u0428' => 'Ш',
            '\u0449' => 'щ', '\u0429' => 'Щ',
            '\u044a' => 'ъ', '\u042a' => 'Ъ',
            '\u044b' => 'ы', '\u042b' => 'Ы',
            '\u044c' => 'ь', '\u042c' => 'Ь',
            '\u044d' => 'э', '\u042d' => 'Э',
            '\u044e' => 'ю', '\u042e' => 'Ю',
            '\u044f' => 'я', '\u042f' => 'Я',

        );

        foreach ($cyr_chars as $cyr_char_key => $cyr_char) {
            $json = str_replace($cyr_char_key, $cyr_char, $json);
        }

        return $json;
    }

    function file_get_contents_curl($url) {
        $data = null;
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
        curl_setopt($ch, CURLOPT_NOSIGNAL, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT_MS, 80);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_TIMEOUT_MS, 500);        
        $data = curl_exec($ch);
        curl_close($ch);

        return $data;
    }

    function ResultData($type,$data)
    {
        mysql_query("DELETE FROM `result` WHERE `type` = '".$type."'");
        mysql_query("INSERT INTO `result` (`type` ,`data`) VALUES ('".$type."',  '".$data."');");
    }
    
    function SendQueryPost($UrlQuery,$arrayPost) {
        $data = null;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
        curl_setopt($ch, CURLOPT_NOSIGNAL, 1);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $UrlQuery);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $arrayPost);       
        $data = curl_exec($ch);
        curl_close($ch);
        return $data;
    }

    function showPeriod($time) {
        return sprintf("%02d:%02d:%02d", (int)($time / 3600), (int)(($time % 3600) / 60), $time % 60);
    }

?>